#include "framework.h"

#include <cmath> //for sqrt (square root) function

//**************************************
#ifdef WIN32
	#include <windows.h>
#else
	#include <sys/time.h>
#endif

long getTime()
{
	#ifdef WIN32
		return GetTickCount();
	#else
		struct timeval tv;
		gettimeofday(&tv,NULL);
		return (int)(tv.tv_sec*1000 + (tv.tv_usec / 1000));
	#endif
}

// **************************************

Point::Point()
{
	x = y = z = 0.0f;
}

Point::Point(float x, float y, float z)
{
	this->x = x;
	this->y = y;
	this->z = z;
}

void Point::Random(float range)
{
	//rand returns a value between 0 and RAND_MAX
	x = (rand() / (double)RAND_MAX) * 2 * range - range; //value between -range and range
	y = (rand() / (double)RAND_MAX) * 2 * range - range; //value between -range and range
	z = (rand() / (double)RAND_MAX) * 2 * range - range; //value between -range and range
}

//***********************************

Line::Line(Point a, Point b)
{
	start = a;
	end = b;
}

float Line::Lenght()
{
	//pitagoras
	return sqrt( (end.x - start.x) * (end.x - start.x) + (end.y - start.y) * (end.y - start.y) );
}

void Line::draw()
{
	glBegin(GL_LINES);
		glVertex2f(start.x, start.y);
		glVertex2f(end.x, end.y);
	glEnd();
}

//*********************************
ClippingRectangle::ClippingRectangle() { xmin = ymin = xmax = ymax = 0.0f; }

ClippingRectangle::ClippingRectangle(float xmin, float ymin, float xmax, float ymax)
{
	this->xmin = xmin;
	this->ymin = ymin;
	this->xmax = xmax;
	this->ymax = ymax;
}

void ClippingRectangle::set(const Point& a, const Point& b)
{
	if (a.y < b.y)
	{
		ymin = a.y;
		ymax = b.y;
	}
	else
	{
		ymin = b.y;
		ymax = a.y;
	}

	if (a.x < b.x)
	{
		xmin = a.x;
		xmax = b.x;
	}
	else
	{
		xmin = b.x;
		xmax = a.x;
	}
}

void ClippingRectangle::draw()
{
	glBegin(GL_LINE_LOOP);
		glVertex2f(xmin, ymin);
		glVertex2f(xmin, ymax);
		glVertex2f(xmax, ymax);
		glVertex2f(xmax, ymin);
	glEnd();	
}

//this function changes the line passed as parameter to match the clipping plane
//it returns false if the line is totally outside
bool ClippingRectangle::clipLine( Line& line )
{
	//Clip the line to be inside the rectangle
	//TO DO...
	return true;
}

//this function returns the bit code of the point according to the clipping rectangle
code ClippingRectangle::computeCode( float x, float y )
{
	//TO DO...
	return 0;
}